/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module at93c46_mix(
    clk          ,
    rst_n        ,
    start        ,
    mode         ,
    addr         ,
    wdata        ,
    rdata        ,
    rdata_vld    ,
    rdy          ,
    do           ,
    di           ,
    cs           ,
    sk             
    );

    parameter           DATA_W  =     8;
    parameter           EWEN    = 2'b00;
    parameter           WRITE   = 2'b01;
    parameter           READ    = 2'b10;

    input               clk      ;
    input               rst_n    ;
    input [6:0]         addr     ;
    input [1:0]         mode     ;
    input               start    ;
    input               do       ;

    input [DATA_W-1:0]  wdata    ;
    output              rdy      ;
    output              di       ;
    output              cs       ;
    output              sk       ;
    output[DATA_W-1:0]  rdata    ;
    output              rdata_vld;

    reg   [DATA_W-1:0]  rdata    ;
    reg                 rdata_vld;
    reg                 rdy      ;
    reg                 di       ;
    reg                 cs       ;
    reg                 sk       ;


    wire                start_vld;
    reg [2:0]           opcode   ;
    reg [17:0]          dout     ;
    reg [1:0]           mode_reg ;

    reg                 flag_work;

    reg[19:0]           cnt0     ;
    wire                add_cnt0 ;
    wire                end_cnt0 ;

    reg[ 4:0]           cnt1     ;
    wire                add_cnt1 ;
    wire                end_cnt1 ;

    reg[ 2:0]           cnt2     ;
    wire                add_cnt2 ;
    wire                end_cnt2 ;
    wire                en_sk0   ;
    wire                en_sk1   ;
    wire                en_di    ;

    reg[19:0]           x        ;
    reg[ 5:0]           y        ;
    reg[ 2:0]           u        ;



    assign start_vld = flag_work==0 && start;

    always  @(*)begin
        if(mode==EWEN)
            opcode = 3'b100;
        else if(mode==WRITE)
            opcode = 3'b101;
        else
            opcode = 3'b110;
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dout <= 0;
        end
        else if(start_vld) begin
            dout <={opcode,addr,wdata}; 
        end
    end


    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            mode_reg <= 0;
        end
        else if(start_vld) begin
            mode_reg <= mode;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            flag_work <= 0;
        end
        else if(start_vld) begin
            flag_work <= 1;
        end
        else if(end_cnt2)begin
            flag_work <= 0;
        end
    end


    
    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt0 <= 0;
        end
        else if(add_cnt0)begin
            if(end_cnt0)
                cnt0 <= 0;
            else
                cnt0 <= cnt0 + 1;
        end
    end

    assign add_cnt0 = flag_work;       
    assign end_cnt0 = add_cnt0 && cnt0== x-1;   

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt1 <= 0;
        end
        else if(add_cnt1)begin
            if(end_cnt1)
                cnt1 <= 0;
            else
                cnt1 <= cnt1 + 1;
        end
    end

    assign add_cnt1 = end_cnt0;       
    assign end_cnt1 = add_cnt1 && cnt1==y-1 ;


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
             cnt2 <= 0;
         end
         else if(add_cnt2)begin
             if(end_cnt2)
                  cnt2 <= 0;
             else
                  cnt2 <= cnt2 + 1;
          end
    end
    
    assign add_cnt2 = end_cnt1;       
    assign end_cnt2 = add_cnt2 && cnt2==u-1;   


    assign en_sk1 = add_cnt0 && cnt0==x/2-1 && cnt2==1-1;
    assign en_sk0 = end_cnt0                            ;
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            sk <= 0;
        end
        else if(add_cnt0 && cnt0==x/2-1 && cnt2==1-1)begin
            sk <= 1;
        end
        else if(end_cnt0)begin
            sk <= 0;
        end
    end

    assign en_di = add_cnt0 && cnt0==1-1 && cnt2==1-1;

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            di <= 0;
        end
        else if(en_di) begin
            di <= dout[17-cnt1];
        end
    end

 
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            cs <= 0;
        end
        else if(start_vld==1 || (add_cnt2 && cnt2==2-1 && end_cnt2==0)) begin
            cs <= 1;
        end
        else if((add_cnt2 && cnt2==1-1) || end_cnt2)begin
            cs <= 0;
        end
    end

 
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            rdata <= 0;
        end
        else if(end_cnt0 && cnt1 >=10 && cnt2==1-1 && mode_reg==READ ) begin
            rdata[17-cnt1] <= do;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            rdata_vld <= 0;
        end
        else begin
            rdata_vld <= end_cnt2 && mode_reg==READ;
        end
    end

    always  @(*)begin
        if(start || flag_work)
            rdy = 1'b0;
        else
            rdy = 1'b1;
    end


    always  @(*)begin
        if(mode_reg==WRITE && cnt2==1-1)begin
            x = 250;
            y =  18;
            u =   3;  
        end
        else if(mode_reg==WRITE && (cnt2==2-1 ))begin
            x = 250 ;
            y =   1 ;
            u =   3 ;  
        end
        else if(mode_reg==WRITE && cnt2==3-1)begin
            x = 500000;  
            y =   1 ;
            u =   3 ;  
        end
        else if(mode_reg==READ && cnt2==1-1)begin
            x = 250;
            y =  18;
            u =   2;  
        end
        else if(mode_reg==READ && cnt2==2-1)begin
            x = 250 ; 
            y =   1 ;
            u =   2 ;  
        end
        else if(mode_reg==EWEN && cnt2==1-1)begin
            x = 250;
            y =  10;
            u =   2;  
        end
        else begin
            x = 250 ;  
            y =   1 ;
            u =   2 ;  
        end
    end


endmodule

