/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

//`define KEY_SCAN
module at93c46_top(
    clk      ,
    rst_n    ,
    key_col  ,
    mo       ,
    cs       ,
    mi       ,
    sk       ,
    key_row  ,
    seg_sel  ,
    seg_data 
    );

    parameter    TIME_1S = 50_000_000;


    input               clk          ;
    input               rst_n        ;
    input [3:0]         key_col      ;
    input               mo           ;

    output              cs           ;
    output              mi           ;
    output              sk           ;
    output[3:0]         key_row      ;
    output[5:0]         seg_sel      ;
    output[7:0]         seg_data     ;

    wire                rdy          ;
    wire                rdata_vld    ;
    wire  [3:0]         key_en       ;

    wire  [23:0]        data_load    ;
    wire                data_load_vld;

    wire  [23:0]        clk_data_out ;

    wire  [6:0]         addr         ;
    wire  [1:0]         mode         ;
    wire                start        ;

    wire  [7:0]         wdata        ;
    wire  [7:0]         rdata        ;




             `ifdef KEY_SCAN
    key_scan  u_key_scan(
                .clk     (clk    ),
                .rst_n   (rst_n  ),
                .key_col (key_col),
                .key_row (key_row),
                .key_en  (key_en )
             );
            `else             
   key_module  u_key_module(
                .clk     (clk     ),
                .rst_n   (rst_n   ),
                .key_in  (~key_col),
                .key_vld (key_en  )
             );

            `endif
    clock_data#(.TIME_1S(TIME_1S)) u_clock_data(
                .clk           (clk           ),
                .rst_n         (rst_n         ),
                .data_load     (data_load     ),
                .data_load_vld (data_load_vld ),
                .key_en        (key_en        ),
                .data_out      (clk_data_out  ) 
    );

    seg_disp#(.SEG_NUM(6)) u_seg_disp(
                .rst_n       (rst_n       ),
                .clk         (clk         ),
                .din         (clk_data_out),
                .din_vld     ({6{1'b1}}   ),
                .seg_sel     (seg_sel     ),
                .segment     (seg_data    ) 
             );
    
    
   data_processor#(.TIME_1S(TIME_1S)) u_data_pro(
                      .clk        (clk         )  ,
                      .rst_n      (rst_n       )  ,
                      .din        (clk_data_out)  ,
                      .start      (start       )  ,
                      .mode       (mode        )  ,
                      .addr       (addr        )  ,
                      .wdata      (wdata       )  ,
                      .rdata      (rdata       )  ,
                      .rdata_vld  (rdata_vld   )  ,
                      .rdy        (rdy         )  ,
                      .dout       (data_load   )  ,
                      .dout_vld   (data_load_vld )    
    
    );


    at93c46_mix u_at93c46_mix(
                   .clk        (clk        )  ,
                   .rst_n      (rst_n      )  ,
                   .start      (start      )  ,
                   .mode       (mode       )  ,
                   .addr       (addr       )  ,
                   .wdata      (wdata      )  ,
                   .rdata      (rdata      )  ,
                   .rdata_vld  (rdata_vld  )  ,
                   .rdy        (rdy        )  ,
                   .do         (mo         )  ,
                   .di         (mi         )  ,
                   .cs         (cs         )  ,
                   .sk         (sk         )    
    );


endmodule

