/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module clock_data(
    clk          ,
    rst_n        ,
    data_load    ,
    data_load_vld,
    key_en       ,
    data_out   
    );

    parameter      TIME_1S    =       50_000_000   ;

    input               clk          ;
    input               rst_n        ;
    input               data_load_vld;
    input [23:0]        data_load    ;
    input [ 3:0]        key_en       ;
    output[23:0]        data_out     ;
    wire  [23:0]        data_out     ;


    reg   [25:0]        cnt_1s       ;
    reg   [3:0]         miao_ge      ;
    reg   [3:0]         miao_shi     ;
    reg   [3:0]         fen_ge       ;
    reg   [3:0]         fen_shi      ;
    reg   [3:0]         shi_ge       ;
    reg   [3:0]         shi_shi      ;
    reg   [2:0]         set_sel      ;
    reg                 set_flag     ;
    wire                add_set_sel  ;
    wire                add_cnt_1s   ;
    wire                add_miao_ge  ;
    wire                add_miao_shi ;
    wire                add_fen_ge   ;
    wire                add_fen_shi  ;
    wire                add_shi_ge   ;
    wire                add_shi_shi  ;
    wire                end_cnt_1s   ;
    wire                end_set_sel  ;
    wire                end_miao_ge  ;
    wire                end_miao_shi ;
    wire                end_fen_ge   ;
    wire                end_fen_shi  ;
    wire                end_shi_ge   ;
    wire                end_shi_shi  ;
    wire                set_miao_ge  ;
    wire                set_miao_shi ;
    wire                set_fen_ge   ;
    wire                set_fen_shi  ;
    wire                set_shi_ge   ;
    wire                set_shi_shi  ;

    reg  [ 3:0]         x            ;
    reg  [ 2:0]         y            ;

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            set_flag <= 1'b0;
        end
        else if(key_en[0]) begin
            set_flag <= ~ set_flag;
        end
    end



    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            set_sel <= 0;
        end
        else if(add_set_sel)begin
            if(end_set_sel)
                set_sel <= 0;
            else
                set_sel <= set_sel + 1;
        end
        else if(set_flag==0)begin
            set_sel <= 0;
        end
    end

    assign add_set_sel = set_flag && key_en[1];       
    assign end_set_sel = add_set_sel && set_sel==6-1 ;   

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt_1s <= 0;
        end
        else if(add_cnt_1s)begin
            if(end_cnt_1s)
                cnt_1s <= 0;
            else
                cnt_1s <= cnt_1s + 1;
        end
    end

    assign add_cnt_1s = set_flag==0 ;       
    assign end_cnt_1s = add_cnt_1s && cnt_1s==TIME_1S-1 ;   





    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            miao_ge <= 0;
        end
        else if(add_miao_ge)begin
            if(end_miao_ge)
                miao_ge <= 0;
            else
                miao_ge <= miao_ge + 1;
        end
        else if(data_load_vld)begin
            miao_ge <=  data_load[3:0];
        end
    end

    assign add_miao_ge = (end_cnt_1s || set_miao_ge) ;       
    assign end_miao_ge = add_miao_ge && miao_ge==10-1 ;
    assign set_miao_ge = set_flag    && set_sel==0 && key_en[2];   



    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            miao_shi <= 0;
        end
        else if(add_miao_shi)begin
            if(end_miao_shi)
                miao_shi <= 0;
            else
                miao_shi <= miao_shi + 1;
        end
        else if(data_load_vld)begin
            miao_shi <= data_load[7:4];
        end
    end

    assign add_miao_shi = (end_miao_ge || set_miao_shi);       
    assign end_miao_shi = add_miao_shi && miao_shi==6-1;
    assign set_miao_shi = set_flag     && set_sel==1 && key_en[2];   


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            fen_ge <= 0;
        end
        else if(add_fen_ge)begin
            if(end_fen_ge)
                fen_ge <= 0;
            else
                fen_ge <= fen_ge + 1;
        end
        else if(data_load_vld)begin
            fen_ge <= data_load[11:8];
        end
    end

    assign add_fen_ge = (end_miao_shi || set_fen_ge);       
    assign end_fen_ge = add_fen_ge && fen_ge==10-1;
    assign set_fen_ge = set_flag     && set_sel==2 && key_en[2];   


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            fen_shi <= 0;
        end
        else if(add_fen_shi)begin
            if(end_fen_shi)
                fen_shi <= 0;
            else
                fen_shi <= fen_shi + 1;
        end
        else if(data_load_vld)begin
            fen_shi <= data_load[15:12];
        end
    end

    assign add_fen_shi = (end_fen_ge || set_fen_shi);       
    assign end_fen_shi = add_fen_shi && fen_shi==6-1;
    assign set_fen_shi = set_flag     && set_sel==3 && key_en[2];   

   always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            shi_ge <= 0;
        end
        else if(add_shi_ge)begin
            if(end_shi_ge)
                shi_ge <= 0;
            else
                shi_ge <= shi_ge + 1;
        end
        else if(data_load_vld)begin
            shi_ge <= data_load[19:16];
        end
    end

    assign add_shi_ge = (end_fen_shi || set_shi_ge);       
    assign end_shi_ge = add_shi_ge && shi_ge==x-1;
    assign set_shi_ge = set_flag     && set_sel==4 && key_en[2];   


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            shi_shi <= 0;
        end
        else if(add_shi_shi)begin
            if(end_shi_shi)
                shi_shi <= 0;
            else
                shi_shi <= shi_shi + 1;
        end
        else if(data_load_vld)begin
            shi_shi <= data_load[23:20];
        end
    end

    assign add_shi_shi = (end_shi_ge || set_shi_shi);       
    assign end_shi_shi = add_shi_shi && shi_shi==y-1;
    assign set_shi_shi = set_flag     && set_sel==5 && key_en[2];   

    always  @(*)begin
        if(shi_shi<2)
            x =  10;
        else
            x =   4;
    end

    always  @(*)begin
        if(set_flag && set_sel==5 && shi_ge>=4) 
            y = 2;
        else
            y = 3;
    end
    assign data_out = {shi_shi,shi_ge,fen_shi,fen_ge,miao_shi,miao_ge};


endmodule

