/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module data_processor(
    clk          ,
    rst_n        ,
    din          ,
    start        ,
    mode         ,
    addr         ,
    wdata        ,
    rdata        ,
    rdata_vld    ,
    rdy          ,
    dout         ,
    dout_vld       
    
    );

    parameter      S_IDLE  =  2'b00     ;
    parameter      S_EWEN  =  2'b01     ;
    parameter      S_READ  =  2'b10     ;
    parameter      S_WRITE =  2'b11     ;
    parameter      EWEN    = 2'b00      ;
    parameter      WRITE   = 2'b01      ;
    parameter      READ    = 2'b10      ;
    parameter      D_LEN   =  3         ;
    parameter      WR_NUM  = (D_LEN+1)  ;
    parameter      RD_NUM  = (D_LEN+1)*2;
    parameter      TIME_1S = 50_000_000 ;

    input               clk             ;
    input               rst_n           ;
    input [D_LEN*8-1:0] din             ;
    output[D_LEN*8-1:0] dout            ;
    output              dout_vld        ;
    output[6:0]         addr            ;
    output[1:0]         mode            ;
    output              start           ;
    output[7:0]         wdata           ;
    input               rdy             ;
    input [7:0]         rdata           ;
    input               rdata_vld       ;

    reg   [1:0]         state_c         ;
    reg   [1:0]         state_n         ;
    reg   [D_LEN*8-1:0] dout            ;
    reg                 dout_vld        ;
    reg   [25:0]        time_cnt        ;
    wire                add_time_cnt    ;
    wire                end_time_cnt    ;
    reg   [ 3:0]        wr_cnt          ;
    wire                add_wr_cnt      ;
    wire                end_wr_cnt      ;
    reg   [ 2:0]        rd_cnt          ;
    wire                add_rd_cnt      ;
    wire                end_rd_cnt      ;
    reg   [ 4:0]        x               ;
    reg   [ 1:0]        mode            ;
    reg                 crc_din_vld     ;
    wire                crc_dout_vld    ;
    wire  [7:0]         crc_dout        ;
    reg                 crc_clr         ;
    reg   [7:0]         crc_din         ;
    reg   [6:0]         addr            ;
    reg                 write_sel       ;
    reg   [7:0]         wdata           ;
    reg                 start           ;
    reg   [8*8-1 :0]    dout_temp       ;
    reg                 dout0_err       ;
    reg                 dout1_err       ;
    wire                ewen2read_start ; 
    wire                read2idle_start ; 
    wire                idle2write_start; 
    wire                write2idle_start; 
    wire                start_read      ;
    reg                 flag_wait_crc   ;
    wire  [D_LEN*8-1 :0]dout0_temp      ;
    wire  [D_LEN*8-1 :0]dout1_temp      ;


    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            state_c <= S_EWEN;
        end
        else begin
            state_c <= state_n;
        end
    end

    always  @(*)begin
        case(state_c)
            S_EWEN : begin
                if(ewen2read_start)begin
                    state_n = S_READ ;
                end
                else begin
                    state_n = state_c;
                end
            end
            S_READ : begin
                if(read2idle_start)begin
                    state_n = S_IDLE;
                end
                else begin
                    state_n = state_c;
                end
            end
            S_IDLE : begin
                if(idle2write_start)begin
                    state_n = S_WRITE;
                end
                else begin
                    state_n = state_c;
                end
            end
            S_WRITE : begin
                if(write2idle_start)begin
                    state_n = S_IDLE;
                end
                else begin
                    state_n = state_c;
                end
            end
            default : begin
                state_n = S_EWEN;
            end
        endcase
    end


    assign   ewen2read_start  = state_c==S_EWEN  && add_time_cnt && time_cnt==1000-1 && rdy;
    assign   read2idle_start  = state_c==S_READ  && end_rd_cnt      ;
    assign   idle2write_start = state_c==S_IDLE  && end_time_cnt    ;
    assign   write2idle_start = state_c==S_WRITE && end_wr_cnt      ;


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            time_cnt <= 0;
        end
        else if(add_time_cnt)begin
            if(end_time_cnt)
                time_cnt <= 0;
            else
                time_cnt <= time_cnt + 1;
        end
    end

    assign add_time_cnt = 1;       
    assign end_time_cnt = add_time_cnt && time_cnt==TIME_1S-1 ;   


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            wr_cnt <= 0;
        end
        else if(add_wr_cnt)begin
            if(end_wr_cnt)
                wr_cnt <= 0;
            else
                wr_cnt <= wr_cnt + 1;
        end
    end

    assign add_wr_cnt = state_c==S_WRITE && rdy; 
    assign end_wr_cnt = add_wr_cnt && wr_cnt == WR_NUM-1 ;


    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            rd_cnt <= 0;
        end
        else if(add_rd_cnt)begin
            if(end_rd_cnt)
                rd_cnt <= 0;
            else
                rd_cnt <= rd_cnt + 1;
        end
    end

    assign add_rd_cnt = state_c==S_READ && crc_dout_vld; 
    assign end_rd_cnt = add_rd_cnt && rd_cnt == RD_NUM-1 ;


    assign start_read = state_c==S_READ && flag_wait_crc==0 && rdy;

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            flag_wait_crc <= 0;
        end
        else if(start_read)begin
            flag_wait_crc <= 1;
        end
        else if(crc_dout_vld)begin
            flag_wait_crc <= 0;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            mode <= EWEN; 
        end
        else if(state_c==S_EWEN) begin
            mode <= EWEN; 
        end
        else if(state_c==S_WRITE)begin
            mode <= WRITE;
        end
        else if(state_c==S_READ)begin
            mode <= READ;
        end
    end



    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            crc_din <= 0;
        end
        else if(add_wr_cnt && end_wr_cnt==0) begin 
            crc_din <= din[8*(D_LEN-wr_cnt)-1 -:8];
        end
        else begin
            crc_din <= rdata;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            crc_din_vld <= 1'b0;
        end
        else if(add_wr_cnt && end_wr_cnt==0) begin
            crc_din_vld <= 1'b1;
        end
        else begin
            crc_din_vld <= rdata_vld;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            crc_clr <= 1'b0;
        end
        else if((add_rd_cnt && rd_cnt==4-1)|| read2idle_start || write2idle_start) begin 
            crc_clr <= 1'b1;
        end
        else begin
            crc_clr <= 1'b0;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr <= 0;
        end
        else if(state_c==S_WRITE) begin
            addr <= {4'b0,write_sel,wr_cnt[1:0]};
        end
        else if(state_c==S_EWEN) begin
            addr <= 7'b1100000;
        end
        else begin
            addr <= {4'b0,rd_cnt};
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            write_sel <= 1'b0;
        end
        else if(write2idle_start) begin
            write_sel <= ~write_sel;
        end
    end

  


    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata <= 0;
        end
        else if(wr_cnt==WR_NUM-1) begin
            wdata <= crc_dout;
        end
        else begin
            wdata <= din[8*(D_LEN-wr_cnt)-1 -:8];
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            start <= 1'b0;
        end
        else if(add_wr_cnt || start_read || ewen2read_start)begin
            start <= 1'b1;
        end
        else begin
            start <= 1'b0;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dout_temp<= 0;
        end
        else if(rdata_vld) begin
            dout_temp[(8-rd_cnt)*8-1 -:8] <= rdata;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dout0_err <= 1'b0;
        end
        else if(add_rd_cnt && rd_cnt==4-1) begin
            if(crc_dout!=0)
                dout0_err <= 1'b1;
            else
                dout0_err <= 1'b0;
        end
    end

    always  @(*)begin
        if(crc_dout!=0)
            dout1_err = 1'b1;
        else
            dout1_err = 1'b0;
    end


    assign dout0_temp = dout_temp[8*8-1 -:24];
    assign dout1_temp = dout_temp[4*8-1 -:24];



    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dout <= 0;
        end
        else if(end_rd_cnt) begin
            if(dout0_err==1'b0 && dout1_err==1'b0)begin
                if(dout0_temp > dout1_temp)
                    dout <= dout0_temp;
                else
                    dout <= dout1_temp;
            end
            else if(dout0_err==1'b0)begin
                dout <= dout0_temp;
            end
            else begin
                dout <= dout1_temp;
            end
        end
    end


    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            dout_vld <= 1'b0;
        end
        else if(end_rd_cnt) begin
            if(dout0_err && dout1_err)begin
                dout_vld <= 1'b0;
            end
            else begin
                dout_vld <= 1'b1;
            end
        end
        else begin
            dout_vld <= 1'b0;
        end
    end


   crc8_d8 u_crc8_d8(
              .clk      (clk          ),
              .rst_n    (rst_n        ),
              .clr      (crc_clr      ),
              .din_vld  (crc_din_vld  ),
              .din      (crc_din      ),
              .dout_vld (crc_dout_vld ),
              .dout     (crc_dout     )      
    );





endmodule

