/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module  key_scan(
                 clk    ,
                 rst_n  ,
                 key_col,
                 key_row,
                 key_en   
               );


    parameter      KEY_W    =   4      ;
    parameter      CHK_COL  =   0      ;
    parameter      CHK_ROW  =   1      ;
    parameter      DELAY    =   2      ;
    parameter      WAIT_END =   3      ;
    parameter      COL_CNT  =   16     ;
    parameter      TIME_20MS=   1000000;

    input               clk              ;
    input               rst_n            ;
    input [3:0]         key_col          ;

    output[3:0]         key_en           ;
    output[KEY_W-1:0]   key_row          ;

    reg   [3:0]         key_out          ;
    reg   [KEY_W-1:0]   key_row          ;
    reg                 key_vld          ;


    reg   [3:0]         key_col_ff0      ;
    reg   [3:0]         key_col_ff1      ;
    reg   [1:0]         key_col_get      ;
    reg   [3:0]         key_en           ;
    wire                end_shake_cnt    ;
    reg                 end_shake_cnt_ff0;
    reg   [3:0]         state_c          ;
    reg   [19:0]        shake_cnt        ;
    reg   [3:0]         state_n          ;
    reg   [1:0]         row_index        ;
    reg   [15:0]        row_cnt          ;
    wire                col2row_start    ; 
    wire                row2del_start    ;
    wire                del2wait_start   ;
    wire                wait2col_start   ;
    wire                add_row_cnt      ;
    wire                end_row_cnt      ;
    wire                add_shake_cnt    ;
    wire                add_row_index    ;
    wire                end_row_index    ;


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_ff0 <= 4'b1111;
        key_col_ff1 <= 4'b1111;
    end
    else begin
        key_col_ff0 <= key_col    ;
        key_col_ff1 <= key_col_ff0;
    end
end


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        shake_cnt <= 0; 
    end
    else if(add_shake_cnt) begin
        if(end_shake_cnt)
            shake_cnt <= 0; 
        else
            shake_cnt <= shake_cnt+1 ;
   end
end
assign add_shake_cnt = key_col_ff1!=4'hf;
assign end_shake_cnt = add_shake_cnt  && shake_cnt == TIME_20MS-1 ;


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        state_c <= CHK_COL;
    end
    else begin
        state_c <= state_n;
    end
end

always  @(*)begin
    case(state_c)
        CHK_COL: begin
                     if(col2row_start )begin
                         state_n = CHK_ROW;
                     end
                     else begin
                         state_n = CHK_COL;
                     end
                 end
        CHK_ROW: begin
                     if(row2del_start)begin
                         state_n = DELAY;
                     end
                     else begin
                         state_n = CHK_ROW;
                     end
                 end
        DELAY :  begin
                     if(del2wait_start)begin
                         state_n = WAIT_END;
                     end
                     else begin
                         state_n = DELAY;
                     end
                 end
        WAIT_END: begin
                     if(wait2col_start)begin
                         state_n = CHK_COL;
                     end
                     else begin
                         state_n = WAIT_END;
                     end
                  end
       default: state_n = CHK_COL;
    endcase
end
assign col2row_start = state_c==CHK_COL  && end_shake_cnt;
assign row2del_start = state_c==CHK_ROW  && row_index==3 && end_row_cnt;
assign del2wait_start= state_c==DELAY    && end_row_cnt;
assign wait2col_start= state_c==WAIT_END && key_col_ff1==4'hf;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_row <= 4'b0;
    end
    else if(state_c==CHK_ROW)begin
        key_row <= ~(1'b1 << row_index);
    end
    else begin
        key_row <= 4'b0;
    end
end





always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        row_index <= 0; 
    end
    else if(add_row_index) begin
        if(end_row_index)
            row_index <= 0; 
        else
            row_index <= row_index+1 ;
   end
   else if(state_c!=CHK_ROW)begin
       row_index <= 0;
   end
end
assign add_row_index = state_c==CHK_ROW && end_row_cnt;
assign end_row_index = add_row_index  && row_index == 4-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        row_cnt <= 0; 
    end
    else if(add_row_cnt) begin
        if(end_row_cnt)
            row_cnt <= 0; 
        else
            row_cnt <= row_cnt+1 ;
   end
end
assign add_row_cnt = state_c==CHK_ROW || state_c==DELAY;
assign end_row_cnt = add_row_cnt  && row_cnt == 16-1 ;



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_get <= 0;
    end
    else if(state_c==CHK_COL && end_shake_cnt ) begin
        if(key_col_ff1==4'b1110)
            key_col_get <= 0;
        else if(key_col_ff1==4'b1101)
            key_col_get <= 1;
        else if(key_col_ff1==4'b1011)
            key_col_get <= 2;
        else 
            key_col_get <= 3;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_out <= 0;
    end
    else if(state_c==CHK_ROW && end_row_cnt)begin
        key_out <= {row_index,key_col_get};
    end
    else begin
        key_out <= 0;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_vld <= 1'b0;
    end
    else if(state_c==CHK_ROW && end_row_cnt && key_col_ff1[key_col_get]==1'b0)begin
        key_vld <= 1'b1;
    end
    else begin
        key_vld <= 1'b0;
    end
end


always  @(*)begin
    if(rst_n==1'b0)begin
        key_en = 0;
    end
    else if(key_vld && key_out==0)begin
        key_en = 4'b0001;
    end
    else if(key_vld && key_out==1)begin
        key_en = 4'b0010;
    end
    else if(key_vld && key_out==2)begin
        key_en = 4'b0100;
    end
    else begin
        key_en = 0;
    end
end


endmodule

