/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module  seg_disp(
    rst_n       ,
    clk         ,
    din         ,
    din_vld     ,
    seg_sel     ,
    segment      
    );



parameter  SEG_WID        =       8           ;
parameter  SEG_NUM        =       8           ;
parameter  COUNT_WID      =       26          ;
parameter  TIME_2MS       =       20'd100000  ;

 
parameter  NUM_0          =       8'b1100_0000;
parameter  NUM_1          =       8'b1111_1001;
parameter  NUM_2          =       8'b1010_0100;
parameter  NUM_3          =       8'b1011_0000;
parameter  NUM_4          =       8'b1001_1001;
parameter  NUM_5          =       8'b1001_0010;
parameter  NUM_6          =       8'b1000_0010;
parameter  NUM_7          =       8'b1111_1000;
parameter  NUM_8          =       8'b1000_0000;
parameter  NUM_9          =       8'b1001_0000;
parameter  NUM_ERR        =       8'b1000_0110;


input                             clk        ;
input                             rst_n      ;
input  [SEG_NUM*4-1:0]            din        ;
input  [SEG_NUM-1:0]              din_vld    ;
output [SEG_NUM-1:0]              seg_sel    ;
output [SEG_WID-1:0]              segment    ;

reg    [SEG_NUM-1:0]              seg_sel    ;
reg    [SEG_WID-1:0]              segment    ;
reg    [COUNT_WID-1:0]            cnt_2ms    ;
wire                              add_cnt_2ms;
wire                              end_cnt_2ms;
reg    [SEG_NUM-1:0]              cnt_sel    ;
wire                              add_cnt_sel;
wire                              end_cnt_sel;
reg    [4*SEG_NUM-1:0]            din_ff0    ;
reg    [        4-1:0]            seg_tmp    ;
integer                           ii         ;



always @(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        cnt_2ms <= 0;
    end
    else if(add_cnt_2ms)begin
        if(end_cnt_2ms)
            cnt_2ms <= 0;
        else
            cnt_2ms <= cnt_2ms + 1;
    end
end

assign add_cnt_2ms = 1;       
assign end_cnt_2ms = add_cnt_2ms && cnt_2ms==TIME_2MS-1 ;   

always @(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        cnt_sel <= 0;
    end
    else if(add_cnt_sel)begin
        if(end_cnt_sel)
            cnt_sel <= 0;
        else
            cnt_sel <= cnt_sel + 1;
    end
end

assign add_cnt_sel = end_cnt_2ms;       
assign end_cnt_sel = add_cnt_sel && cnt_sel== SEG_NUM-1;   



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        seg_sel <= {SEG_NUM{1'b1}};
    end
    else begin
        seg_sel <= ~(1'b1 << cnt_sel);
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        din_ff0 <= 0;
    end
    else begin
        for(ii=0;ii<SEG_NUM;ii=ii+1)begin
            if(din_vld[ii]==1'b1)begin
                din_ff0[(ii+1)*4-1 -:4] <= din[(ii+1)*4-1 -:4];
            end
            else begin
                din_ff0[(ii+1)*4-1 -:4] <= din_ff0[(ii+1)*4-1 -:4];
            end
        end
    end
end

always  @(*)begin
    seg_tmp = din_ff0[(cnt_sel+1)*4-1 -:4]; 
end


always@(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        segment<=NUM_0;
    end
    else begin
        case(seg_tmp)
            4'd0:segment <= NUM_0;
            4'd1:segment <= NUM_1;
            4'd2:segment <= NUM_2;
            4'd3:segment <= NUM_3;
            4'd4:segment <= NUM_4;
            4'd5:segment <= NUM_5;
            4'd6:segment <= NUM_6;
            4'd7:segment <= NUM_7;
            4'd8:segment <= NUM_8;
            4'd9:segment <= NUM_9;
            default:begin
                segment <= NUM_ERR;
            end
        endcase
    end
end

endmodule

